import json
import logging
import os
import os.path as osp
from dataclasses import dataclass, field
from datetime import datetime

import hydra
from hydra import compose, initialize
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron.annotations import AnnotationsCompiler
from zendron.comments import Comment, CommentCompiler
from zendron.items import get_annotated_attachments, get_attachments, get_metadatas
from zendron.metadata import Metadata, MetadataCompiler

log = logging.getLogger(__name__)


@dataclass
class UserCitationKey:
    metadata: Metadata
    pod_path: str
    _metadata_wiki: str = None
    _comments_wiki: str = None
    _annotations_wiki: str = None

    @property
    def metadata_wiki(self):
        if self._metadata_wiki is None:
            alias = "metadata"
            link = f"dendron://{self.metadata.project_dir}/{self.metadata.dendron_limb}.{self.metadata.title_dendron}"
            self._metadata_wiki = f"[[{alias}|{link}]]"
        return self._metadata_wiki

    @property
    def comments_wiki(self):
        if self._comments_wiki is None:
            alias = "comments"
            link = f"dendron://{self.metadata.project_dir}/{self.metadata.dendron_limb}.{self.metadata.title_dendron}.comments"
            self._metadata_wiki = f"[[{alias}|{link}]]"
        return self._metadata_wiki

    @property
    def annotations_wiki(self):
        if self._annotations_wiki is None:
            alias = "annotations"
            link = f"dendron://{self.metadata.project_dir}/{self.metadata.dendron_limb}.{self.metadata.title_dendron}.annotations"
            self._metadata_wiki = f"[[{alias}|{link}]]"
        return self._metadata_wiki

    # only currently support one attachment which is annotations. Need to refactor metadata adding properties for the methods, or initializing in post_init.


class UserCitationKeyCompiler:
    def __init__(self, user_citation_key: UserCitationKey):
        self.user_citation_key = user_citation_key
        self.metadata = user_citation_key.metadata
        self.pod_path = user_citation_key.pod_path
        self.lines: list = None

    def compile(self) -> str:
        lines = []
        lines.append("## Metadata\n\n***\n")
        lines.append(f"{self.user_citation_key.metadata_wiki}")
        lines.append(f"!{self.user_citation_key.metadata_wiki}")
        lines.append("\n***\n")
        lines.append("## Comments\n\n***\n")
        lines.append(f"{self.user_citation_key.comments_wiki}")
        lines.append(f"!{self.user_citation_key.comments_wiki}")
        lines.append("\n***\n")
        lines.append("## Annotations\n\n***\n")
        lines.append(f"{self.user_citation_key.annotations_wiki}")
        lines.append(f"!{self.user_citation_key.annotations_wiki}")
        lines.append("\n***")
        self.lines = lines
        return lines

    def write(self):
        path = osp.join(self.pod_path, "user")
        os.makedirs(path, exist_ok=True)
        file_path = osp.join(path, f"{self.metadata.citation_key}.md")
        with open(file_path, "w") as f:
            for line in self.lines:
                f.write(line)
                f.write("\n")


@hydra.main(
    version_base=None, config_path=osp.join(os.getcwd(), "conf"), config_name="config"
)
def main(cfg: DictConfig):
    pass


if __name__ == "__main__":
    main()



# %%

from zendron import sync

# # %%

# # https://github.com/urschrei/pyzotero/issues/148

# group_id = 4761702
# library_type = "group" #or 'user'
# group_name = "parameter_estimation"
# # From Key generation: Your new API key has been created and is displayed below. Please save it now as it will not be accessible again after this point.
# api_key = "jzPvsy3TRlC8blUsbBZRMUMK"

# zot = zotero.Zotero(group_id, library_type, api_key)
# items = zot.top(limit=5)

# %%
# all_annotations = zot.everything(zot.items(itemType="annotation"))
# %%
# TODO implement tag update in vscode
# zot.add_tags(all_annotations[0], "test-tag")
# # adding tags
# all_annotations[0]['data']['tags']
# zot.update_item(all_annotations[0])
# %%
# all_annotations[0]
# %%
# TODO update only recently modified
# modified_dates = [i['data']['dateModified'] for i in all_annotations]
# modified_dates_parsed = [utc_parse(i) for i in modified_dates]
# LAST_MODIFIED_DATE = max(modified_dates_parsed)
# LAST_MODIFIED_DATE = max(modified_dates)



import json
import logging
import os
import os.path as osp
import typing
from typing import Union

from pyzotero import zotero
from pyzotero.zotero_errors import ResourceNotFound

from zendron.annotations import Annotation
from zendron.comments import Comment
from zendron.items import get_annotated_attachments, get_attachments, get_metadatas
from zendron.metadata import Metadata

log = logging.getLogger(__name__)


class Cache:
    def __init__(self, zot: zotero.Zotero = None, **kwargs):
        self.zot = zot
        self._cache_dir = "zendron_cache"
        if not osp.exists(self._cache_dir):
            os.makedirs(self._cache_dir)
        self._cache_file_path = None
        self._data = None

    @property
    def data(self):
        if self._data is None:
            self._data = {}
        return self._data

    @property
    def cache_dir(self):
        if self._cache_dir is None:
            self._cache_dir = "zendron_cache"
            if not osp.exists(self._cache_dir):
                os.makedirs(self._cache_dir)
        return self._cache_dir

    @property
    def cache_file_path(self):
        if self._cache_file_path is None:
            self._cache_file_path = osp.join(self.cache_dir, "cache.json")
        return self._cache_file_path

    def load(self):
        if not osp.exists(self.cache_file_path):
            with open(self.cache_file_path, "w") as f:
                json.dump(self.data, f, indent=4)
        with open(self.cache_file_path, "r") as f:
            self._data = json.load(f)

    def add_metadata(self, metadata: Metadata):
        self.data[metadata.key] = {
            "version": metadata.metadata["data"]["version"],
            "title_dendron": metadata.title_dendron,
            "citation_key": metadata.citation_key,
            "annotated_attachments": {},
            "comment": {},
        }

    def add_annotated_attachment(self, attach_zen: dict):
        attachment = self.zot.item(attach_zen["key"])
        self.data[attachment["data"]["parentItem"]]["annotated_attachments"][
            attachment["key"]
        ] = {
            "version": attachment["version"],
            "title_dendron": attach_zen["zendron_title"],
            "annotations": {},
        }

    def add_annotations(self, annotations: list[dict]):
        # Disgusting nested loop 🤮...just refactor with the zot maybe?
        for meta in self.data.keys():
            if self.data[meta]["annotated_attachments"]:
                for attach in self.data[meta]["annotated_attachments"].keys():
                    for annot in annotations:
                        if annot["data"]["parentItem"] == attach:
                            self.data[meta]["annotated_attachments"][attach][
                                "annotations"
                            ][annot["key"]] = {
                                "version": annot["version"],
                            }

    def add_comment(self, comment: Comment):
        for meta in self.data.keys():
            if meta == comment.parentItem:
                self.data[meta]["comment"] = {
                    "version": comment.version,
                }

    def write(self):
        with open(self.cache_file_path, "w") as f:
            json.dump(self.data, f, indent=4)

    def update_metadata(self, metadata: Metadata):
        self.data[metadata.key]["version"] = metadata.metadata["data"]["version"]
        self.data[metadata.key]["title_dendron"] = metadata.title_dendron
        self.data[metadata.key]["citation_key"] = metadata.citation_key
        self.write()

    def update_annotated_attachment(self, attach_zen: dict):
        attachment = self.zot.item(attach_zen["key"])
        self.data[attachment["data"]["parentItem"]]["annotated_attachments"][
            attachment["key"]
        ]["version"] = attachment["version"]
        self.data[attachment["data"]["parentItem"]]["annotated_attachments"][
            attachment["key"]
        ]["title_dendron"] = attach_zen["zendron_title"]
        self.write()

    def delete_metadata(self, key: str):
        del self.data[key]
        self.write()


def main():
    pass


if __name__ == "__main__":
    main()



import logging
from typing import Union

from pyzotero import zotero

log = logging.getLogger(__name__)


def get_metadatas(
    zot: zotero.Zotero, collection: str = None, item_types: list = None
) -> list:
    metadata_list = []
    if collection is None:
        for i in zot.everything(zot.items()):
            if i["data"]["itemType"] in item_types:
                metadata_list.append(i)

    if collection is not None:
        collection_key = [
            i for i in zot.collections() if i["data"]["name"] == collection
        ][0]["key"]
        items = zot.collection_items(collection_key)
        for i in items:
            if i["data"]["itemType"] in item_types:
                metadata_list.append(i)

    log.info("Extracting Data Per entry")
    return metadata_list


def get_annotated_attachments(
    zot: zotero.Zotero, meta_key: Union[list[str], str]
) -> list[dict]:
    annotated_attachments = []
    if isinstance(meta_key, str):
        meta_key = [meta_key]
    for item in meta_key:
        data = zot.children(item)
        for item in data:
            try:
                if (
                    item["data"]["itemType"] == "attachment"
                    and item["data"]["contentType"] == "application/pdf"
                ):
                    data_temp = {}
                    data_temp["key"] = item["key"]
                    data_temp["version"] = item["version"]
                    data_temp["title"] = item["data"]["title"]
                    data_temp["zendron_title"] = set_title(
                        item["data"], item["data"]["title"]
                    )
                    annotated_attachments.append(data_temp)
            except KeyError:
                log.info("Ignoring attachments that aren't 'application/pdf'")
    return annotated_attachments


def get_attachments(zot: zotero.Zotero, meta_key: Union[list[str], str]) -> list[dict]:
    attachments = []
    data = zot.children(meta_key)
    for item in data:
        data_temp = {}
        data_temp["key"] = item["key"]
        data_temp["itemType"] = item["data"]["itemType"]
        try:
            data_temp["title"] = item["data"]["title"]
        except KeyError:
            data_temp["title"] = None
        data_temp["zendron_title"] = set_title(item["data"], data_temp["title"])
        attachments.append(data_temp)
    return attachments


# TODO remove if it works as a method in cache and it isn't being used anywhere else... might useful to hold onto
# def get_annotations(zot: zotero.Zotero, metadatas: list[dict]) -> list[dict]:
#     annotations = []
#     for meta in metadatas:
#         annotated_attachments = get_annotated_attachments(zot, meta["key"])
#         annotations_temp = []
#         for attach in annotated_attachments:
#             annotations_temp.extend(zot.children(attach["key"]))
#         annotations.extend(annotations_temp)
#     return annotations


def get_comments(
    zot: zotero.Zotero, metadata_list: list[dict], attachments: list[dict]
) -> list[dict]:
    comments = []
    attachments = []
    for metadata in metadata_list:
        attachments.extend(get_attachments(zot, metadata["key"]))

    comments = []
    comments.extend(
        [
            item
            for item in attachments
            if item["itemType"] == "note" and item["zendron_title"] == "zendron comment"
        ]
    )
    return comments


def set_title(data: dict, title: str) -> str:
    # def set_title(title: str = None, item_type: str = None) -> str:
    if data["itemType"] == "note":
        if data["note"] == "zendron comment":
            title = "zendron comment"
            return title
        else:
            return None
    prefix = title.split("_")[0]
    # if "supp", "supplementary", "supp.", "supplementary material", "supplementary materials", sup, SI, in prefix then set_title = prefix
    if prefix in [
        "supp",
        "Supp",
        "supplementary",
        "supp.",
        "supplementary-material",
        "supplementary-materials",
        "sup",
        "SI",
    ]:
        title = prefix
    else:
        title = "annotations"
    return title


def main():
    pass


if __name__ == "__main__":
    main()



from datetime import datetime
from typing import Union


def utc_parse(date_string):
    # substitute Z with +00:00 for proper UTC formatting. Z stands for 0 offset.
    if date_string == "":
        return None
    if date_string[-1] == "Z":
        date_string = date_string[:-1] + "+00:00"
    return datetime.fromisoformat(date_string)


def main():
    pass


if __name__ == "__main__":
    main()



import json
import logging
import os
import os.path as osp
from typing import Union

from pyzotero import zotero
from pyzotero.zotero_errors import ResourceNotFound

from zendron.items import get_annotated_attachments, get_attachments, get_metadatas

log = logging.getLogger(__name__)

from abc import ABC, abstractmethod, abstractproperty


class Cache(ABC):
    def __init__(self, zot: zotero.Zotero = None, **kwargs):
        self.zot = zot
        self._cache_dir = "zendron_cache"
        self._cache_file_path = None

    @property
    def cache_dir(self):
        if self._cache_dir is None:
            self._cache_dir = "zendron_cache"
            if not osp.exists(self._cache_dir):
                os.makedirs(self._cache_dir)
        return self._cache_dir

    def load(self):
        if osp.exists(self.cache_file_path):
            with open(self.cache_file_path, "r") as f:
                cache_temp = json.load(f)
            cache = cache_temp.copy()
            for k in cache_temp.keys():
                try:
                    self.zot.item(k)
                except ResourceNotFound:
                    log.info("Items have been deleted from Zotero.")
                    del cache[k]
            with open(self.cache_file_path, "w") as f:
                json.dump(cache, f, indent=4)
            return cache
        else:
            cache = {}
            return cache

    @abstractproperty
    def cache_file_path(self):
        pass

    @abstractmethod
    def initialize(self) -> dict:
        pass

    @abstractmethod
    def update(self) -> dict:
        pass


class MetadataCache(Cache):
    def __init__(
        self,
        zot: zotero.Zotero = None,
        item_types: list = None,
        **kwargs,
    ):
        super().__init__(zot, **kwargs)
        self.item_types = (" || ").join(item_types)

    @property
    def cache_file_path(self):
        if self._cache_file_path is None:
            self._cache_file_path = osp.join(self.cache_dir, "metadata_cache.json")
        return self._cache_file_path

    def initialize(self, metadata_list: list[dict]) -> dict:
        with open(self.cache_file_path, "w") as f:
            cache = self.zot.item_versions(itemType=self.item_types)
            cache = self._metadata_filter(cache, metadata_list)
            json.dump(cache, f, indent=4)
            return cache

    def update(self, metadata_list: list[dict]) -> dict:
        with open(self.cache_file_path, "w") as f:
            cache = self.zot.item_versions(itemType=self.item_types)
            cache = self._metadata_filter(cache, metadata_list)
            json.dump(cache, f, indent=4)

    def _metadata_filter(self, cache: dict, metadata_list: list[dict]):
        meta_keys = [item["key"] for item in metadata_list]
        cache = {key: cache[key] for key in meta_keys}
        return cache


class AnnotationsCache(Cache):
    def __init__(
        self,
        zot: zotero.Zotero = None,
        **kwargs,
    ):
        super().__init__(zot, **kwargs)
        self.item_type = ["annotation"]

    @property
    def cache_file_path(self):
        if self._cache_file_path is None:
            self._cache_file_path = osp.join(self.cache_dir, "annotations_cache.json")
        return self._cache_file_path

    def initialize(self, metadatas: list[dict]) -> dict:
        with open(osp.join(self.cache_dir, "annotations_cache.json"), "w") as f:
            cache = self.zot.item_versions(itemType=self.item_type)
            annotations = self._get_annotations(self.zot, metadatas)
            cache = self._annotations_filter(cache, annotations)
            json.dump(cache, f, indent=4)
            return cache

    def update(self, metadatas: list[dict]) -> dict:
        with open(self.cache_file_path, "w") as f:
            cache = self.zot.item_versions(itemType=self.item_type)
            annotations = self._get_annotations(self.zot, metadatas)
            cache = self._annotations_filter(cache, annotations)
            json.dump(cache, f, indent=4)

    def _annotations_filter(self, cache: dict, annotations: list[dict]):
        annot_keys = [item["key"] for item in annotations]
        cache = {key: cache[key] for key in annot_keys}
        return cache

    def _get_annotations(self, zot: zotero.Zotero, metadatas: list[dict]) -> list[dict]:
        annotations = []
        for meta in metadatas:
            annotated_attachments = get_annotated_attachments(zot, meta["key"])
            annotations_temp = []
            for attach in annotated_attachments:
                annotations_temp.extend(zot.children(attach["key"]))
            annotations.extend(annotations_temp)
        return annotations


class CommentsCache(Cache):
    def __init__(
        self,
        zot: zotero.Zotero = None,
        **kwargs,
    ):
        super().__init__(zot, **kwargs)
        self.cache_file = "comments_cache.json"

    def initialize(self, comments: list[dict]) -> dict:
        itemType = ["note"]
        with open(osp.join(self.cache_dir, "comments_cache.json"), "w") as f:
            cache = self.zot.item_versions(itemType=itemType)
            comment_keys = [item["key"] for item in comments]
            cache = {key: cache[key] for key in comment_keys}
            json.dump(cache, f, indent=4)
        pass

    def update(self) -> dict:
        pass

    def load(self) -> dict:
        pass


# IDEA class cache?
def init(name: str = None):
    if name == "annotations":
        if osp.exists("zendron_cache/annotations_cache.json"):
            with open("zendron_cache/annotations_cache.json", "r") as f:
                cache = json.load(f)
        else:
            cache = {}
        return cache
    elif name == "metadata":
        if osp.exists("zendron_cache/metadata_cache.json"):
            with open("zendron_cache/metadata_cache.json", "r") as f:
                cache = json.load(f)
        else:
            cache = {}
        return cache
    elif name == "comments":
        if osp.exists("zendron_cache/comments_cache.json"):
            with open("zendron_cache/comments_cache.json", "r") as f:
                cache = json.load(f)
        else:
            cache = {}
        pass



import json
import logging
import os
import os.path as osp
from dataclasses import dataclass, field
from datetime import datetime

import hydra
from hydra import compose, initialize
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron.annotations import AnnotationsCompiler
from zendron.cache import Cache
from zendron.comments import Comment, CommentCompiler
from zendron.items import (
    get_annotated_attachments,
    get_attachments,
    get_comments,
    get_metadatas,
)
from zendron.metadata import Metadata, MetadataCompiler
from zendron.user_citation_key import UserCitationKey, UserCitationKeyCompiler

log = logging.getLogger(__name__)


def main(cfg: DictConfig):
    api_key = cfg.api_key
    library_id = cfg.library_id
    library_type = cfg.library_type
    zot = zotero.Zotero(library_id, library_type, api_key)
    cache = Cache(zot)
    metadata_list = get_metadatas(zot, cfg.collection, cfg.item_types)
    for metadata in tqdm(metadata_list):
        attachments = get_attachments(zot, metadata["key"])
        metadata = Metadata(
            metadata,
            attachments,
            cfg.dendron_limb,
        )
        meta_compiler = MetadataCompiler(metadata, cfg.pod_path)
        meta_compiler.compile()
        meta_compiler.write()
        user_citation_key = UserCitationKey(metadata, cfg.pod_path)
        user_citation_key_compiler = UserCitationKeyCompiler(user_citation_key)
        user_citation_key_compiler.compile()
        user_citation_key_compiler.write()
        cache.add_metadata(metadata)
        # TODO rename to something like annotated_attachments. This is not a list of the annotations themselves, but the attachment that contains the annotations.
        annotated_attachments = get_annotated_attachments(zot, metadata.key)
        for attach in annotated_attachments:
            cache.add_annotated_attachment(attach)
            annotations = zot.children(attach["key"])
            annot_compiler = AnnotationsCompiler(
                attach["zendron_title"],
                metadata.title_dendron,
                cfg.local_image_path,
                cfg.dendron_limb,
                cfg.pod_path,
            )
            annot_compiler.compile(annotations)
            annot_compiler.write()
            cache.add_annotations(annotations)
        comment = Comment(zot, attachments)
        comment_compiler = CommentCompiler(
            comment, metadata.title_dendron, cfg.dendron_limb
        )
        comment_compiler.compile(comment)
        comment_compiler.write_comment()
        cache.add_comment(comment)
        cache.write()


if __name__ == "__main__":
    main()



import bibtexparser
import hydra
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero


def get_doi(zot):
    zot.add_parameters(format="bibtex")
    x = zot.item("C46JTY9H")
    doi = x.entries[0]["doi"]
    # x is a BibDatabase object.
    # Look up BibDatabase methods from bibtextparser documentation
    return doi


@hydra.main(
    version_base=None, config_path=osp.join(os.getcwd(), "conf"), config_name="config"
)
def main(cfg: DictConfig):
    api_key = cfg.api_key
    library_id = cfg.library_id
    library_type = cfg.library_type  # or 'user'
    zot = zotero.Zotero(library_id, library_type, api_key)
    doi = get_doi(zot)
    print(f"retrieved DOI from journalArticle... What is it like to be a Bat?: {doi}")


if __name__ == "__main__":
    main()



# %%
import glob
import json
import logging
import os
import os.path as osp
from dataclasses import dataclass, field
from datetime import datetime

import frontmatter
import hydra
from html2text import html2text
from hydra import compose, initialize
from markdown import markdown
from markdownify import markdownify as md
from omegaconf import DictConfig, OmegaConf
from pyzotero import zotero
from tqdm import tqdm

from zendron import front
from zendron.annotations import AnnotationsCompiler
from zendron.metadata import Metadata, MetadataCompiler

# @dataclass
# class Comment:
#     zot: zotero.Zotero = None
#     dendron_limb: str = None
#     zotero_comment_title: str = None
#     _formatted_title: str = None

#     @property
#     def format_zotero_comment_title(self) -> str:
#         if self._formatted_title is None:
#             self._formatted_title = f"{self.zotero_comment_title}\n"
#         return self._formatted_title

#     def title_check(self, note: str = None) -> str:
#         zotero_title = self.format_zotero_comment_title(self.zotero_comment_title)
#         extracted_title = note[: len(zotero_title)]
#         return zotero_title == extracted_title


# def load_comment():
#     comment_paths = glob.glob(osp.join("notes", f"{dendron_limb}.*.comments.md"))
#     leaf_paths = glob.glob(osp.join("notes", f"{dendron_limb}.*.*.md"))
#     possible_paths = glob.glob(osp.join("notes", f"{dendron_limb}.*.md"))
#     meta_paths = list(set(possible_paths) - set(leaf_paths))
#     for path in meta_paths:
#         child = zotero_comment(path, zot, self.zotero_comment_title)
#         if child is not None:
#             note = child["data"]["note"]
#             comment_title = format_zotero_comment_title(self.zotero_comment_title)
#             zotero_note = last_note[len(comment_title) :]
#     return last_note


# def zotero_comment(
#     meta_path: str,
#     zot: zotero.Zotero = None,
#     zotero_comment_title: str = None,
# ) -> dict:
#     data = frontmatter.load(meta_path)
#     children = zot.children(data["metadata_key"])
#     for child in children:
#         if child["data"]["itemType"] == "note" and title_check(
#             zotero_comment_title, child["data"]["note"]
#         ):
#             return child
#     return None


# def format_zotero_comment_title(zotero_comment_title) -> str:
#     formatted_title = f"{zotero_comment_title}\n"
#     return formatted_title


# def sync_comments(
#     zot: zotero.Zotero = None,
#     dendron_limb: str = None,
#     zotero_comment_title: str = None,
# ):
#     paths = glob.glob(osp.join("notes", f"{dendron_limb}.*.comments.md"))
#     for path in paths:
#         meta_note_path = (path.split(".")[:-2]) + ["md"]
#         meta_note_path = ".".join(meta_note_path)
#         data = frontmatter.load(meta_note_path)

#         with open(path, "r") as f:
#             text = f.read()
#             text = format_zotero_comment_title(zotero_comment_title) + text
#         note = zot.item_template("note")
#         # No conversion to HTML for easy Zotero reading.
#         note["note"] = text
#         comment = zotero_comment(zot, dendron_limb, zotero_comment_title)
#         if comment is not None and comment["data"]["note"] != text:
#             comment["note"] = text
#             zot.update_item(comment)
#         else:
#             zot.create_items([note], data["metadata_key"])


# def to_file(markdown):
#     with open("delete.md", "w") as f:
#         f.write(markdown)


@hydra.main(
    version_base=None, config_path=osp.join(os.getcwd(), "conf"), config_name="config"
)
def main(cfg: DictConfig):
    api_key = cfg.api_key
    library_id = cfg.library_id
    library_type = cfg.library_type  # or 'user'
    dendron_limb = cfg.dendron_limb
    zot = zotero.Zotero(library_id, library_type, api_key)
    raw_text = sync_comments(
        zot, dendron_limb, zotero_comment_title=cfg.zotero_comment_title
    )
    text = read_comments(zot, dendron_limb)
    to_file(text)

    # comment = Comment()
    # comment_compiler = CommentCompiler()
    # comment_compiler.compile()
    # comment_compiler.write_comment()


if __name__ == "__main__":
    main()
